#include "kanji_union.h"

SJIS_TABLE sjis_table;
SJIS_TABLE *u = &sjis_table;

BOOL ms932_sjis_file_in_current_directory(char *buffer, int buffer_size) {
    BOOL b = TRUE;
    FILE *file = fopen(MS932_SJIS_BIN, "rb"); /* current directory */
    if(file == NULL) {
        b = FALSE;
        memset(buffer, 0, buffer_size);
    } else {
        fclose(file);
        strcpy(buffer, MS932_SJIS_BIN);
    }

    return b;
}

BOOL ms932_sjis_file_in_installed_directory(char *buffer, int buffer_size) {
    BOOL b = TRUE;
    FILE *file = fopen(C_MS932_SJIS_BIN, "rb"); /* directory where dayu was installed. */
    if(file == NULL) {
        b = FALSE;
        memset(buffer, 0, buffer_size);
    } else {
        fclose(file);
        strcpy(buffer, C_MS932_SJIS_BIN);
    }

    return b;
}

BOOL ms932_sjis_file_in_path_directory(char *buffer, int buffer_size) {
    BOOL b = TRUE;
    char *data = malloc(BUFFER_SIZE);
    if(data == NULL) {
        fprintf(stderr, "malloc() error-10: %d bytes", BUFFER_SIZE);
        b = FALSE;
        return b;
    }

    strcpy(data, getenv("Path") );
    path_name(data, MS932_SJIS_BIN, buffer, buffer_size); /* directory included in Path */

    if( strlen(buffer) == 0 ) {
        b = FALSE;
    }

    free(data);
    return b;
}

BOOL ms932_sjis_file_in_specified_directory(char *buffer, int buffer_size) {
    BOOL b = TRUE;
    FILE *file = fopen(u->binary_name, "rb"); /* current directory */
    if(file == NULL) {
        b = FALSE;
        memset(buffer, 0, buffer_size);
    } else {
        fclose(file);
        strcpy(buffer, u->binary_name);
    }

    return b;
}

BOOL setting(SJIS_TABLE *u, BOOL no_searching, BOOL *which_table) {
    char buffer[BLOCK_SIZE];
    char c[DOUBLE_KANJI];
    FILE *file;
    int buffer_size = BLOCK_SIZE;
    int i, index;
    unsigned char uc;
    OPEN_CLOSE_FUNCTION *open_close = malloc( sizeof(OPEN_CLOSE_FUNCTION) * SEARCH_COUNT );
    if(open_close == NULL) {
        fprintf(stderr, "malloc() error-11: %d bytes", sizeof(OPEN_CLOSE_FUNCTION) * SEARCH_COUNT );
        return FALSE;
    }

    open_close[0] = ms932_sjis_file_in_installed_directory;
    open_close[1] = ms932_sjis_file_in_current_directory;
    open_close[2] = ms932_sjis_file_in_path_directory;
    open_close[3] = ms932_sjis_file_in_specified_directory;

    for(i = 0; i < SEARCH_COUNT; ++i) {
        if( open_close[i](buffer, buffer_size) ) {
            if(*which_table) {
                fprintf(stdout, "%s is primarily accessible to dayu.\n", buffer);
                *which_table = FALSE;
            }

            break;
        } else {
            switch(i) {
            case 0:
                if(no_searching) {
                    fprintf(stderr, "open_close[%d]() error: %s\n", i, MS932_SJIS_BIN);
                    free(open_close);
                    return FALSE;
                }

                break;
            case 3:
                fprintf(stderr, "open_close[%d]() error: %s\n", i, MS932_SJIS_BIN);
                free(open_close);
                return FALSE;
            default:
                break;
            }

        }

    }

    u->size = 0xFC4C * SJIS_KANJI_SIZE_1;
    u->table = malloc(u->size);
    if(u->table == NULL) {
        fprintf(stderr, "malloc() error-12: %d bytes\n", u->size);
        free(open_close);
        return FALSE;
    }

    file = fopen(buffer, "rb");
    for(i = 1; i <= RECORD_COUNT; ++i) {
        fread(c, DOUBLE_KANJI, 1, file);
        if(i < FIRST_KANJI) {
            uc = (unsigned char)c[0];
            index = (int)uc;
            populated(u, index, 0, c[SJIS_KANJI_SIZE]);
        } else {
            populated(u,  inverted(c), 0, c[SJIS_KANJI_SIZE]);
            populated(u,  inverted(c), 1, c[ SJIS_KANJI_SIZE + 1 ]);
        }

    }

    fclose(file);
    free(open_close);
    return TRUE;
}

char *growing(SJIS_TABLE *u, int index) {
    int k = index * SJIS_KANJI_SIZE_1;
    char *p = u->table + k;
    strcpy(u->combo, p);
    return u->combo;
}

unsigned char working(SJIS_TABLE *u, int index, int place) {
    int k = index * SJIS_KANJI_SIZE_1 + place;
    char *p = u->table + k;
    unsigned char c = (unsigned char) *p;
    return c;
}

void clear_table(SJIS_TABLE *u) {
    memset(u->table, 0, u->size);
}

void detach_table(SJIS_TABLE *u) {
    free(u->table);
}

void populated(SJIS_TABLE *u, int index, int place, char c) {
    int k = index * SJIS_KANJI_SIZE_1 + place;
    *(u->table + k) = c;
}

